import { defineStore } from "pinia";
import { useAbilityStore } from "@/stores/ability";
import router from "@/router";
import type { PermissionSection } from "../../types/permissionTypes";

export type navigationModel = {
  [key in topLevelNavigationType]: navigationSplitModel;
};

export interface navigationSplitModel {
  topTitle?: string;
  top?: Array<navigationLinkModel>;
  mainTitle: string;
  main: Array<navigationLinkModel>;
}

export type topLevelNavigationType = PermissionSection;

export interface topLevelNavigationModel {
  key: topLevelNavigationType;
  title: string;
  levelDefault: string;
  showSidebar?: boolean;
}

export interface navigationLinkModel {
  key: string;
  title: string;
}

export const useNavigationStore = defineStore("navigation", {
  state: () => {
    return {
      activeNavigation: "operation" as topLevelNavigationType,
      activeLink: null as null | string,
      topLevel: [] as Array<topLevelNavigationModel>,
      navigation: {} as navigationModel,
    };
  },
  getters: {
    activeNavigationObject: (state) => (state.navigation[state.activeNavigation] ?? {}) as navigationSplitModel,
    activeTopLevelObject: (state) =>
      (state.topLevel.find((elem) => elem.key == state.activeNavigation) ?? {}) as topLevelNavigationModel,
  },
  actions: {
    resetNavigation() {
      this.$reset();
    },
    updateTopLevel() {
      const abilityStore = useAbilityStore();
      this.topLevel = [
        ...(abilityStore.canSection("read", "operation")
          ? [
              {
                key: "operation",
                title: "Einsätze",
                levelDefault: "mission",
                showSidebar: false,
              } as topLevelNavigationModel,
            ]
          : []),
        ...(abilityStore.canSection("read", "configuration")
          ? [
              {
                key: "configuration",
                title: "Konfiguration",
                levelDefault: "force",
                showSidebar: true,
              } as topLevelNavigationModel,
            ]
          : []),
        ...(abilityStore.canSection("read", "management")
          ? [
              {
                key: "management",
                title: "Verwaltung",
                levelDefault: "user",
                showSidebar: true,
              } as topLevelNavigationModel,
            ]
          : []),
      ];
      if (this.topLevel.findIndex((e) => e.key == this.activeNavigation) == -1) {
        this.activeNavigation = this.topLevel[0]?.key ?? "operation";
        router.push({ name: `admin-${this.topLevel[0]?.key ?? "operation"}-default` });
      }
    },
    updateNavigation() {
      const abilityStore = useAbilityStore();
      this.navigation = {
        operation: {
          mainTitle: "Einsätze",
          main: [...(abilityStore.can("read", "operation", "mission") ? [{ key: "mission", title: "Einsätze" }] : [])],
        },
        configuration: {
          mainTitle: "Konfiguration",
          main: [
            ...(abilityStore.can("read", "configuration", "force") ? [{ key: "force", title: "Kräfte" }] : []),
            ...(abilityStore.can("read", "configuration", "equipment") ? [{ key: "equipment", title: "Geräte" }] : []),
            ...(abilityStore.can("read", "configuration", "vehicle") ? [{ key: "vehicle", title: "Fahrzeuge" }] : []),
          ],
        },
        management: {
          mainTitle: "Verwaltung",
          main: [
            ...(abilityStore.can("read", "management", "user") ? [{ key: "user", title: "Benutzer" }] : []),
            ...(abilityStore.can("read", "management", "role") ? [{ key: "role", title: "Rollen" }] : []),
            ...(abilityStore.can("read", "management", "backup") ? [{ key: "backup", title: "Backups" }] : []),
            ...(abilityStore.isAdmin() ? [{ key: "import", title: "Datenübernahme" }] : []),
            ...(abilityStore.isAdmin() ? [{ key: "settings", title: "Einstellungen" }] : []),
            ...(abilityStore.isAdmin() ? [{ key: "version", title: "Version" }] : []),
          ],
        },
      } as navigationModel;
      if (
        this.activeNavigationObject.main.findIndex((e) => e.key == this.activeLink) == -1 ||
        this.activeLink == "default"
      ) {
        let link = this.activeNavigationObject.main[0].key;
        router.push({ name: `admin-${this.activeNavigation}-${link}` });
      }
    },
  },
});
