import { defineStore } from "pinia";
import { useConnectionStore } from "./connection";
import * as Y from "yjs";
import { Awareness, type Editor, type EditorState } from "@/helpers/awareness";

export const useMissionDetailStore = defineStore("missionDetail", {
  state: () => {
    return {
      yDoc: new Y.Doc(),
      awareness: new Awareness(),
      docId: null as null | string,
      lastUpdateTimestamp: 0 as number,
      connectionStatus: "disconnected", // 'disconnected', 'connecting', 'connected', 'syncing', 'synced'
    };
  },
  actions: {
    initialize(docId: string) {
      this.docId = docId;

      this.lastUpdateTimestamp = this.loadLastUpdateFromLocalStorage();

      this.setupSocketHandlers();
      this.setupYjsObservers();
      this.setupAwarenessObservers();
    },

    setupSocketHandlers() {
      const connectionStore = useConnectionStore();

      connectionStore.connection?.on("package-sync", (data) => {
        try {
          this.connectionStatus = "syncing";

          Y.applyUpdate(this.yDoc, new Uint8Array(data.update));

          if (data.timestamp > this.lastUpdateTimestamp) {
            this.lastUpdateTimestamp = data.timestamp;
            this.saveLastUpdateToLocalStorage();
          }

          this.connectionStatus = "synced";
        } catch (error) {
          console.error("Error applying update:", error);
          this.requestFullSync();
        }
      });
      connectionStore.connection?.on("sync-get-missing-updates", (data) => {
        const clientUpdates = Y.encodeStateAsUpdate(this.yDoc, new Uint8Array(data.stateVector));

        connectionStore.connection?.emit("mission:sync-client-updates", {
          update: clientUpdates,
          timestamp: Date.now(),
        });
      });

      connectionStore.connection?.on("package-init-awareness", (data) => {
        if (data.type == "editors") {
          let update = data.update as Array<[string, Editor]>;
          for (const e of update) {
            if (connectionStore.connection?.id == e[0]) return;
            this.awareness.updateEditor(e[0], {
              action: "update",
              data: e[1],
            });
          }
        } else if (data.type == "state") {
          let update = data.update as Array<[string, EditorState]>;
          for (const e of update) {
            if (connectionStore.connection?.id == e[0]) return;
            this.awareness.updateEditorState(e[0], {
              action: "update",
              data: e[1],
            });
          }
        }
      });
      connectionStore.connection?.on("package-sync-awareness", (data) => {
        if (connectionStore.connection?.id == data.socketId) return;
        if (data.type == "editors") {
          this.awareness.updateEditor(data.socketId, {
            action: data.action,
            data: data.update,
          });
        } else if (data.type == "state") {
          this.awareness.updateEditorState(data.socketId, {
            action: data.action,
            data: data.update,
          });
        }
      });

      connectionStore.connection?.on("disconnect", () => {
        this.awareness.reset();
      });

      connectionStore.connection?.on("status-mission:join", (data) => {
        console.log("status-mission:join", data);
      });
      connectionStore.connection?.on("status-mission:sync-client-updates", (data) => {
        console.log("status-mission:sync-client-updates", data);
      });
      connectionStore.connection?.on("status-mission:sync-get-missing-updates", (data) => {
        console.log("status-mission:sync-get-missing-updates", data);
      });
      connectionStore.connection?.on("status-mission:sync-client-updates", (data) => {
        console.log("status-mission:sync-client-updates", data);
      });

      this.joinDocument();
    },

    setupYjsObservers() {
      this.yDoc.on("update", (update) => {
        const connectionStore = useConnectionStore();
        if (connectionStore.connected) {
          connectionStore.connection?.emit("mission:sync-client-updates", {
            update: Uint8Array.from(update),
            timestamp: Date.now(),
          });
        }

        this.lastUpdateTimestamp = Date.now();
        this.saveLastUpdateToLocalStorage();
      });
    },

    setupAwarenessObservers() {
      this.awareness.emitter.on("update", (update: { data: EditorState }) => {
        const connectionStore = useConnectionStore();
        connectionStore.connection?.emit("mission:sync-client-awareness", update.data);
      });
    },

    joinDocument() {
      if (!this.docId) return;

      const connectionStore = useConnectionStore();
      connectionStore.connection?.emit("mission:join", this.docId, {
        timestamp: this.lastUpdateTimestamp,
      });
    },

    requestFullSync() {
      if (!this.docId) return;

      const connectionStore = useConnectionStore();
      connectionStore.connection?.emit("mission:join", this.docId, null);
    },

    loadLastUpdateFromLocalStorage() {
      if (!this.docId) return 0;

      const stored = localStorage.getItem(`yjsDoc_timestamp`);
      return stored ? parseInt(stored, 10) : 0;
    },

    saveLastUpdateToLocalStorage() {
      localStorage.setItem(`yjsDoc_timestamp`, this.lastUpdateTimestamp.toString());
    },

    cleanup() {
      this.yDoc.destroy();
      this.yDoc = new Y.Doc();
      this.awareness.destroy();
      this.awareness = new Awareness();

      this.lastUpdateTimestamp = 0;
      localStorage.removeItem("yjsDoc_timestamp");

      const connectionStore = useConnectionStore();
      connectionStore.connection?.emit("mission:leave");

      this.connectionStatus = "disconnected";
    },
  },
});
